<?php
/**
 *
 * SIMPLE Journal-to-feed converter
 *
 * This script converts any DimensioneX game's journal (plain text format, proprietary) into a standard RSS feed.
 * The resulting RSS feed can be then piped via twitterfeed.com or hootsuite.com for great Web 2.0 reuse (Facebook pages, Twitter, you name it)
 *
 * Version 1.1 SIMPLE
 *
 * For DimensioneX 6.4.9 or later
 *
 * http://www.dimensionex.net
 *
 * NOTICE ON THIS SOFTWARE
 *
 * License: GPL 2.0, Open Source, feel free to share
 * Guarantees: None, use at your own risk
 *
 */

// CONFIG HERE
$gamewebsite = 'http://www.gamesite.net/'; // URL to game's official web site (used in RSS feed)
$gamedescr = 'Your great MMORPG game'; // (brief)  game description to be used in game RSS feed
$journalfile = 'http://www.dimensionex.net/journal/journal.txt'; // Path to journal file. Can be an URL starting with http:// OR local file path (absolute or relative) eg. ../private/journal5.txt 
$feedtitle = 'MMORPG Game Feed'; // Human-readable title for Game events RSS Feed
$feedurl = 'http://www.gamesite.net/grss.php'; // URL to Game events RSS Feed (this script)
$debugmode = FALSE; // Set to TRUE to see diagnostic messages
// STOP CONFIG!

header("Expires: Mon, 26 Jul 1997 05:00:00 GMT"); // Date in the past
header("Cache-Control: no-store, no-cache, must-revalidate");
header("Pragma: no-cache");

if ($debugmode) {
	echo '<!doctype html>
	<html xmlns:fb="http://www.facebook.com/2008/fbml">
	';
} else {
 	echo '<?xml version="1.0" encoding="UTF-8"?>'."\n";
 }


error_reporting(0); // Disable session warnings

?>
<rss version="2.0"
	xmlns:content="http://purl.org/rss/1.0/modules/content/"
	xmlns:wfw="http://wellformedweb.org/CommentAPI/"
	xmlns:dc="http://purl.org/dc/elements/1.1/"
	xmlns:atom="http://www.w3.org/2005/Atom"
	xmlns:sy="http://purl.org/rss/1.0/modules/syndication/"
	xmlns:slash="http://purl.org/rss/1.0/modules/slash/"
	>

<channel>
	<title><?php echo $feedtitle; ?></title>
	<atom:link href="<?php echo $feedurl; ?>" rel="self" type="application/rss+xml" />
	<link><?php echo $gamewebsite; ?></link>
	<description><?php $gamedescr; ?></description>
<?php

	// Open journal file
	if (substr($journalfile,0,7)=='http://') { // Fetch from network
		$contents = load($journalfile);
	} else { // Read from disk
		$contents = @file_get_contents($journalfile,FALSE);
	}
	
	
	if (!$contents) { 
		echo "Journal file: '$journalfile' does not exist"; 
		exit(); 
	}

	$mtime = filemtime($journalfile);
	$gmdate_mod = gmdate('D, d M Y H:i:s O', $mtime);
	
	echo "	<lastBuildDate>$gmdate_mod</lastBuildDate>
	<language>en</language>
	<sy:updatePeriod>hourly</sy:updatePeriod>
	<sy:updateFrequency>1</sy:updateFrequency>
";

	$convert = explode("\n", $contents); //create array separate by new line

	for ($i=0;$i<count($convert);$i++) {
		$line_of_text = $convert[$i];
		if ($line_of_text) {
			$parts = explode('|', $line_of_text);

			if (count($parts)>3) {
				$p = $parts[3];
				
				/*
				$desc = $p;
				foreach(mb_list_encodings() as $chr){
						echo mb_convert_encoding($desc, 'UTF-8', $chr)." : ".$chr."\n";
				} 
				*/
				
				$p = mb_convert_encoding($p, 'UTF-8', 'ISO-8859-1');
				$p = str_replace('\n',"<br />\n",$p);
				$whatplain = strip_tags($p);
				
				$link = strip_tags($parts[2]);
				if ($link != $parts[2] || !$link) $link = $gamewebsite; // Sanitize
				
				$ititle = mb_convert_encoding($parts[1], 'UTF-8', 'ISO-8859-1');
				if (($ititle != $prevititle) && ($whatplain != $prevplain)) { // Avoid double lines
					print "<item>
				<title>$ititle</title>
				<link>{$link}</link>
				<pubDate>{$parts[0]}</pubDate>
				<description><![CDATA[$whatplain]]></description>
					<content:encoded><![CDATA[{$p}]]></content:encoded>

				";
				
					$cats = explode('*', trim($parts[4])); // Try separating categories with *
					if (count($cats)<2) // Zero or one categories. Try separating with commas
						$cats = explode(',', trim($parts[4]));
					foreach ($cats as $c) {
						echo "<category><![CDATA[$c]]></category>\n";
					}
					print "</item>\n";
				}
				$prevtitle = $ititle;
				$prevplain = $whatplain;
			}
		}
	}

	fclose($file_handle);	



?>
	</channel>
</rss>


<?

// Functions

/**
 * See http://www.bin-co.com/php/scripts/load/
 * Version : 1.00.A
 */
function load($url,$options=array('method'=>'get','return_info'=>false)) {
    $url_parts = parse_url($url);
    $info = array(//Currently only supported by curl.
        'http_code'    => 200
    );
    $response = '';

    $send_header = array(
        'Accept' => 'text/*',
        'User-Agent' => 'BinGet/1.00.A (http://www.bin-co.com/php/scripts/load/)'
    );

    ///////////////////////////// Curl /////////////////////////////////////
    //If curl is available, use curl to get the data.
    if(function_exists("curl_init")
                and (!(isset($options['use']) and $options['use'] == 'fsocketopen'))) { //Don't user curl if it is specifically stated to user fsocketopen in the options
        if(isset($options['method']) and $options['method'] == 'post') {
            $page = $url_parts['scheme'] . '://' . $url_parts['host'] . $url_parts['path'];
        } else {
            $page = $url;
        }

        $ch = curl_init($url_parts['host']);

        curl_setopt($ch, CURLOPT_URL, $page);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true); //Just return the data - not print the whole thing.
        curl_setopt($ch, CURLOPT_HEADER, true); //We need the headers
        curl_setopt($ch, CURLOPT_NOBODY, false); //The content - if true, will not download the contents
        if(isset($options['method']) and $options['method'] == 'post' and $url_parts['query']) {
            curl_setopt($ch, CURLOPT_POST, true);
            curl_setopt($ch, CURLOPT_POSTFIELDS, $url_parts['query']);
        }
        //Set the headers our spiders sends
        curl_setopt($ch, CURLOPT_USERAGENT, $send_header['User-Agent']); //The Name of the UserAgent we will be using ;)
        $custom_headers = array("Accept: " . $send_header['Accept'] );
        if(isset($options['modified_since']))
            array_push($custom_headers,"If-Modified-Since: ".gmdate('D, d M Y H:i:s \G\M\T',strtotime($options['modified_since'])));
        curl_setopt($ch, CURLOPT_HTTPHEADER, $custom_headers);

        curl_setopt($ch, CURLOPT_COOKIEJAR, "cookie.txt"); //If ever needed...
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, 1);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);

        if(isset($url_parts['user']) and isset($url_parts['pass'])) {
            $custom_headers = array("Authorization: Basic ".base64_encode($url_parts['user'].':'.$url_parts['pass']));
            curl_setopt($ch, CURLOPT_HTTPHEADER, $custom_headers);
        }

        $response = curl_exec($ch);
        $info = curl_getinfo($ch); //Some information on the fetch
        curl_close($ch);

    //////////////////////////////////////////// FSockOpen //////////////////////////////
    } else { //If there is no curl, use fsocketopen
        if(isset($url_parts['query'])) {
            if(isset($options['method']) and $options['method'] == 'post')
                $page = $url_parts['path'];
            else
                $page = $url_parts['path'] . '?' . $url_parts['query'];
        } else {
            $page = $url_parts['path'];
        }

        $fp = fsockopen($url_parts['host'], 80, $errno, $errstr, 30);
        if ($fp) {
            $out = '';
            if(isset($options['method']) and $options['method'] == 'post' and isset($url_parts['query'])) {
                $out .= "POST $page HTTP/1.1\r\n";
            } else {
                $out .= "GET $page HTTP/1.0\r\n"; //HTTP/1.0 is much easier to handle than HTTP/1.1
            }
            $out .= "Host: $url_parts[host]\r\n";
            $out .= "Accept: $send_header[Accept]\r\n";
            $out .= "User-Agent: {$send_header['User-Agent']}\r\n";
            if(isset($options['modified_since']))
                $out .= "If-Modified-Since: ".gmdate('D, d M Y H:i:s \G\M\T',strtotime($options['modified_since'])) ."\r\n";

            $out .= "Connection: Close\r\n";

            //HTTP Basic Authorization support
            if(isset($url_parts['user']) and isset($url_parts['pass'])) {
                $out .= "Authorization: Basic ".base64_encode($url_parts['user'].':'.$url_parts['pass']) . "\r\n";
            }

            //If the request is post - pass the data in a special way.
            if(isset($options['method']) and $options['method'] == 'post' and $url_parts['query']) {
                $out .= "Content-Type: application/x-www-form-urlencoded\r\n";
                $out .= 'Content-Length: ' . strlen($url_parts['query']) . "\r\n";
                $out .= "\r\n" . $url_parts['query'];
            }
            $out .= "\r\n";

            fwrite($fp, $out);
            while (!feof($fp)) {
                $response .= fgets($fp, 128);
            }
            fclose($fp);
        }
    }

    //Get the headers in an associative array
    $headers = array();

    if($info['http_code'] == 404) {
        $body = "";
        $headers['Status'] = 404;
    } else {
        //Seperate header and content
        $separator_position = strpos($response,"\r\n\r\n");
        $header_text = substr($response,0,$separator_position);
        $body = substr($response,$separator_position+4);

        foreach(explode("\n",$header_text) as $line) {
            $parts = explode(": ",$line);
            if(count($parts) == 2) $headers[$parts[0]] = chop($parts[1]);
        }
    }

    if($options['return_info']) return array('headers' => $headers, 'body' => $body, 'info' => $info);
    return $body;
}


?>